import uuid

import barker.schemas.device as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy import delete, select
from sqlalchemy.exc import SQLAlchemyError

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.device import Device
from ..models.login_history import LoginHistory
from ..schemas.user_token import UserToken


router = APIRouter()


@router.put("/{id_}", response_model=schemas.Device)
def update_route(
    id_: uuid.UUID,
    data: schemas.DeviceIn,
    user: UserToken = Security(get_user, scopes=["devices"]),
) -> schemas.Device:
    try:
        with SessionFuture() as db:
            item: Device = db.execute(select(Device).where(Device.id == id_)).scalar_one()
            item.name = data.name
            item.enabled = data.enabled
            item.section_id = data.section.id_
            db.commit()
            return device_info(item)
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


@router.delete("/{id_}")
def delete_route(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["add-devices"]),
):
    try:
        with SessionFuture() as db:
            db.execute(delete(LoginHistory).where(LoginHistory.device_id == id_))
            db.execute(delete(Device).where(Device.id == id_))
            db.commit()
            return {}
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


@router.get("/list", response_model=list[schemas.Device])
def show_list(user: UserToken = Depends(get_user)):
    with SessionFuture() as db:
        return [device_info(item) for item in db.execute(select(Device).order_by(Device.name)).scalars().all()]


@router.get("/{id_}", response_model=schemas.Device)
def show_id(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["devices"]),
) -> schemas.Device:
    with SessionFuture() as db:
        item: Device = db.execute(select(Device).where(Device.id == id_)).scalar_one()
        return device_info(item)


def device_info(item: Device) -> schemas.Device:
    return schemas.Device(
        id=item.id,
        name=item.name,
        enabled=item.enabled,
        section={"id": item.section_id, "name": item.section.name},
        creationDate=item.creation_date,
        lastUser=item.login_history[0].user.name if len(item.login_history) else "Never",
        lastDate=item.login_history[0].date if len(item.login_history) else None,
    )
