import { Component, OnInit } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { ToCsvService } from '../shared/to-csv.service';

import { BeerSaleExportHeaderInterface } from './beer-sale-export-header-interface';
import { BeerSaleReport } from './beer-sale-report';
import { BeerSaleReportDataSource } from './beer-sale-report-datasource';

@Component({
  selector: 'app-beer-sale-report',
  templateUrl: './beer-sale-report.component.html',
  styleUrls: ['./beer-sale-report.component.css'],
})
export class BeerSaleReportComponent implements OnInit {
  info: BeerSaleReport = new BeerSaleReport();
  dataSource: BeerSaleReportDataSource = new BeerSaleReportDataSource(this.info.data);
  form: FormGroup<{
    startDate: FormControl<Date>;
    finishDate: FormControl<Date>;
    regular: FormControl<boolean>;
    happy: FormControl<boolean>;
    staff: FormControl<boolean>;
    nc: FormControl<boolean>;
  }>;

  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns: string[] = ['date'];

  constructor(private route: ActivatedRoute, private router: Router, private toCsv: ToCsvService) {
    // Create form
    this.form = new FormGroup({
      startDate: new FormControl(new Date(), { nonNullable: true }),
      finishDate: new FormControl(new Date(), { nonNullable: true }),
      regular: new FormControl<boolean>(true, { nonNullable: true }),
      happy: new FormControl<boolean>(true, { nonNullable: true }),
      staff: new FormControl<boolean>(true, { nonNullable: true }),
      nc: new FormControl<boolean>(true, { nonNullable: true }),
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { info: BeerSaleReport };
      this.info = data.info;
      this.displayedColumns = ['date'].concat(this.info.headers);
      this.form.setValue({
        startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
        finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate(),
        regular: this.info.regular,
        happy: this.info.happy,
        staff: this.info.staff,
        nc: this.info.nc,
      });
      this.dataSource = new BeerSaleReportDataSource(this.info.data);
    });
  }

  show() {
    const info = this.getInfo();
    this.router.navigate(['beer-sale-report'], {
      queryParams: {
        startDate: info.startDate,
        finishDate: info.finishDate,
        regular: info.regular,
        happy: info.happy,
        staff: info.staff,
        nc: info.nc,
      },
    });
  }

  getInfo(): BeerSaleReport {
    const formModel = this.form.value;

    return new BeerSaleReport({
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY'),
      regular: formModel.regular,
      happy: formModel.happy,
      staff: formModel.staff,
      nc: formModel.nc,
    });
  }

  exportCsv() {
    const init: BeerSaleExportHeaderInterface = { Date: 'date' };
    const headers = this.info.headers.reduce((a, c) => {
      a[c] = c;
      return a;
    }, init);
    const csvData = new Blob([this.toCsv.toCsv(headers, this.info.data)], {
      type: 'text/csv;charset=utf-8;',
    });
    const link = document.createElement('a');
    link.href = window.URL.createObjectURL(csvData);
    link.setAttribute('download', 'beer-sale-report.csv');
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }
}
