import { Component, OnInit } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { environment } from '../../environments/environment';
import { AuthService } from '../auth/auth.service';
import { ToasterService } from '../core/toaster.service';

import { SettingsService } from './settings.service';

@Component({
  selector: 'app-settings',
  templateUrl: './settings.component.html',
  styleUrls: ['./settings.component.css'],
})
export class SettingsComponent implements OnInit {
  prefillCustomerDiscount = true;
  version: string;
  maintenanceForm: FormGroup<{
    startDate: FormControl<Date>;
    finishDate: FormControl<Date>;
  }>;

  beerFile: File | null = null;
  saleFile: File | null = null;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private toaster: ToasterService,
    public auth: AuthService,
    private ser: SettingsService,
  ) {
    this.version = environment.version;
    this.maintenanceForm = new FormGroup({
      startDate: new FormControl(new Date(), { nonNullable: true }),
      finishDate: new FormControl(new Date(), { nonNullable: true }),
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as {
        prefillCustomerDiscount: boolean;
      };

      this.prefillCustomerDiscount = data.prefillCustomerDiscount;
    });
  }

  togglePrefill() {
    this.ser.setPrefillCustomerDiscount(!this.prefillCustomerDiscount).subscribe((x) => {
      this.prefillCustomerDiscount = x;
    });
  }

  detectBeer(event: Event) {
    this.beerFile = ((event.target as HTMLInputElement).files as FileList)[0];
  }

  detectSale(event: Event) {
    this.saleFile = ((event.target as HTMLInputElement).files as FileList)[0];
  }

  runMaintenance() {
    const formModel = this.maintenanceForm.value;
    const startDate = moment(formModel.startDate).format('DD-MMM-YYYY');
    const finishDate = moment(formModel.finishDate).format('DD-MMM-YYYY');

    if (!this.beerFile || !this.saleFile) {
      this.toaster.show('Danger', 'Please choose both files first!');
      return;
    }
    this.ser.runMaintenance(startDate, finishDate, this.beerFile, this.saleFile).subscribe(
      () => {
        this.toaster.show('Success', 'Maintenance done');
      },
      (error) => {
        this.toaster.show('Danger', error);
      },
    );
  }
}
