import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';

import { ReportingLevel } from '../../core/reporting-level';
import { SettleOption } from '../../core/settle-option';
import { ToasterService } from '../../core/toaster.service';
import { VoucherType } from '../../sales/bills/voucher-type';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';
import { SettleOptionService } from '../settle-option.service';

@Component({
  selector: 'app-settle-option-detail',
  templateUrl: './settle-option-detail.component.html',
  styleUrls: ['./settle-option-detail.component.css'],
})
export class SettleOptionDetailComponent implements OnInit, AfterViewInit {
  @ViewChild('nameElement', { static: true }) nameElement?: ElementRef;
  form: FormGroup<{
    name: FormControl<string>;
    voucherType: FormControl<number>;
    reportingLevel: FormControl<number>;
    hasReason: FormControl<boolean>;
  }>;

  item: SettleOption = new SettleOption();
  voucherTypes: { id: number; name: string }[];
  reportingLevel: { id: number; name: string }[];

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private toaster: ToasterService,
    private ser: SettleOptionService,
  ) {
    // Create form
    this.form = new FormGroup({
      name: new FormControl('', { nonNullable: true }),
      voucherType: new FormControl(0, { nonNullable: true }),
      reportingLevel: new FormControl(0, { nonNullable: true }),
      hasReason: new FormControl(false, { nonNullable: true }),
    });
    this.voucherTypes = Object.keys(VoucherType)
      .filter((e) => !isNaN(+e))
      .map((o) => ({ id: +o, name: VoucherType[+o] as string }));
    this.reportingLevel = Object.keys(ReportingLevel)
      .filter((e) => !isNaN(+e))
      .map((o) => ({ id: +o, name: ReportingLevel[+o] as string }));
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { item: SettleOption };
      this.showItem(data.item);
    });
  }

  showItem(item: SettleOption) {
    this.item = item;
    this.form.setValue({
      name: this.item.name,
      voucherType: this.item.voucherType,
      reportingLevel: this.item.reportingLevel,
      hasReason: this.item.hasReason,
    });
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.nameElement !== undefined) {
        this.nameElement.nativeElement.focus();
      }
    }, 0);
  }

  save() {
    this.ser.saveOrUpdate(this.getItem()).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/settle-options');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  delete() {
    this.ser.delete(this.item.id).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/settle-options');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: { title: 'Delete Settle Option?', content: 'Are you sure? This cannot be undone.' },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): SettleOption {
    const formModel = this.form.value;
    this.item.name = formModel.name ?? '';
    this.item.voucherType = formModel.voucherType ?? 0;
    this.item.reportingLevel = formModel.reportingLevel ?? 0;
    this.item.hasReason = formModel.hasReason ?? false;
    return this.item;
  }
}
