import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { MenuCategory } from '../../core/menu-category';
import { Product } from '../../core/product';
import { SaleCategory } from '../../core/sale-category';
import { ToasterService } from '../../core/toaster.service';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';
import { TemporalProductService } from '../temporal-product.service';

@Component({
  selector: 'app-product-detail',
  templateUrl: './temporal-product-detail.component.html',
  styleUrls: ['./temporal-product-detail.component.css'],
})
export class TemporalProductDetailComponent implements OnInit, AfterViewInit {
  @ViewChild('name', { static: true }) nameElement?: ElementRef;
  form: FormGroup<{
    id: FormControl<string>;
    name: FormControl<string>;
    units: FormControl<string>;
    menuCategory: FormControl<string>;
    saleCategory: FormControl<string>;
    price: FormControl<number>;
    hasHappyHour: FormControl<boolean>;
    isNotAvailable: FormControl<boolean>;
    quantity: FormControl<number>;
    validFrom: FormControl<Date | null>;
    validTill: FormControl<Date | null>;
  }>;

  menuCategories: MenuCategory[] = [];
  saleCategories: SaleCategory[] = [];
  item: Product = new Product();

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private toaster: ToasterService,
    private ser: TemporalProductService,
  ) {
    // Create form
    this.form = new FormGroup({
      id: new FormControl<string>('', { nonNullable: true }),
      name: new FormControl<string>('', { nonNullable: true }),
      units: new FormControl<string>('', { nonNullable: true }),
      menuCategory: new FormControl<string>('', { nonNullable: true }),
      saleCategory: new FormControl<string>('', { nonNullable: true }),
      price: new FormControl<number>(0, { nonNullable: true }),
      hasHappyHour: new FormControl<boolean>(false, { nonNullable: true }),
      isNotAvailable: new FormControl<boolean>(false, { nonNullable: true }),
      quantity: new FormControl<number>(0, { nonNullable: true }),
      validFrom: new FormControl(new Date()),
      validTill: new FormControl(new Date()),
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as {
        item: Product;
        menuCategories: MenuCategory[];
        saleCategories: SaleCategory[];
      };
      this.menuCategories = data.menuCategories;
      this.saleCategories = data.saleCategories;
      this.showItem(data.item);
    });
  }

  showItem(item: Product) {
    this.item = item;
    this.form.setValue({
      id: this.item.id ?? '',
      name: this.item.name,
      units: this.item.units,
      menuCategory: this.item.menuCategory?.id ?? '',
      saleCategory: this.item.saleCategory?.id ?? '',
      price: this.item.price,
      hasHappyHour: this.item.hasHappyHour,
      isNotAvailable: this.item.isNotAvailable,
      quantity: this.item.quantity,
      validFrom: this.item.validFrom === null ? null : moment(this.item.validFrom, 'DD-MMM-YYYY').toDate(),
      validTill: this.item.validTill === null ? null : moment(this.item.validTill, 'DD-MMM-YYYY').toDate(),
    });
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.nameElement !== undefined) {
        this.nameElement.nativeElement.focus();
      }
    }, 0);
  }

  update() {
    this.ser.update(this.getItem()).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/temporal-products');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  delete() {
    this.ser.delete(this.item.versionId as string).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/temporal-products');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: { title: 'Delete Product?', content: 'Are you sure? This cannot be undone.' },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): Product {
    const formModel = this.form.value;
    this.item.id = formModel.id;
    this.item.name = formModel.name ?? '';
    this.item.units = formModel.units ?? '';
    if (this.item.menuCategory === null || this.item.menuCategory === undefined) {
      this.item.menuCategory = new MenuCategory();
    }
    this.item.menuCategory.id = formModel.menuCategory;
    if (this.item.saleCategory === null || this.item.saleCategory === undefined) {
      this.item.saleCategory = new SaleCategory();
    }
    this.item.saleCategory.id = formModel.saleCategory;
    this.item.price = formModel.price ?? 0;
    this.item.hasHappyHour = formModel.hasHappyHour ?? false;
    this.item.isNotAvailable = formModel.isNotAvailable ?? false;
    this.item.quantity = formModel.quantity ?? 0;
    this.item.validFrom = !formModel.validFrom ? null : moment(formModel.validFrom).format('DD-MMM-YYYY');
    console.log(formModel.validTill);
    this.item.validTill = !formModel.validTill ? null : moment(formModel.validTill).format('DD-MMM-YYYY');

    return this.item;
  }
}
